----------------------------------------------------------------------------------
-- COPYRIGHT 2015 Jess Eduardo Mndez Rosales
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
-- Descripcin: Este es un contador de 0 a 99 utilizando dos displays de ctodo comn
-- Para iniciar el conteo poner a INI a '1'
-- Para detener el conteo poner a INI a '0'
-- Para reiniciar el conteo poner a RESET a '1'
--
-- NOTA: Se puede cambiar el tiempo del conteo modificando la constante DELAY_FIN
--			Si se utilizan displays de nodo comn entonces negar los vetores del codificador
--       con la operacin lgica NOT.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity CONTA_0_99 is

PORT( CLK        : IN  STD_LOGIC; 					      -- RELOJ DE FPGA 50MHZ
		INI        : IN  STD_LOGIC; 							-- BIT DE INICIO
		RESET 	  : IN  STD_LOGIC;					 	   -- BIT DE RESET
		DISPLAY    : OUT STD_LOGIC_VECTOR(6 DOWNTO 0);  -- VECTOR DE 7 BITS QUE CONTROLAR LOS DISPLAYS
		TRANSISTOR : OUT STD_LOGIC_VECTOR(1 DOWNTO 0)   -- VECTOR DEE 2 BITS QUE CONTROLAR EL SWITCHEO DE 																		-- LOS TRANSISTORES
	 );

end CONTA_0_99;

architecture Behavioral of CONTA_0_99 is

TYPE MAQUINA IS (UNIDADES,DECENAS);       --DECLARACIN DE 
SIGNAL EDO_P,EDO_F : MAQUINA := UNIDADES; --LA MQUINA DE ESTADOS

CONSTANT CONTA_RETRASO_FIN 			  : INTEGER := 49_999_999;							  --CONSTANTE QUE TIENE EL VALOR PARA UN RETRASO DE 10mS
																											  --QUE SE UTILIZAR PARA EL SWITCHEO DE LOS TRANSISTORES
CONSTANT CONTA_SW_FIN      			  : INTEGER := 499_999;																											  
SIGNAL   CONTA_UNIDADES,CONTA_DECENAS : INTEGER RANGE 0 TO 9 := 0; 					  --CONTADORES DE 0-9 PARA UNIDADES Y DECENAS
SIGNAL   CONTA_RETRASO 					  : INTEGER RANGE 0 TO CONTA_RETRASO_FIN := 0; --CONTADOR PARA EL RETRASO DE 10 mS																											  --E INCREMENTAR EL CONTADOR PRINCIPAL
SIGNAL   CONTADOR_PRINCIPAL 			  : INTEGER RANGE 0 TO 9 := 0;                 --CONTADOR PRINCIPAL QUE 
SIGNAL   CONTA_SWITCH 					  : INTEGER RANGE 0 TO CONTA_SW_FIN :=0;

begin



------------PROCESO QUE SE ENCARGA DEL CONTEO---------------------
PROCESS(CLK)
BEGIN
	IF RISING_EDGE(CLK) THEN
		IF RESET = '1' THEN --CONDICIN PARA REINICIAR EL CONTEO CUANDO RESET = '1'
			CONTA_UNIDADES <= 0;
			CONTA_DECENAS <= 0;
		ELSE
			IF INI = '1' THEN --CONDICIN PARA QUE INICIE EL CONTEO, CUANDO INI = '1'
				CONTA_RETRASO <= CONTA_RETRASO +1;
				IF CONTA_RETRASO = CONTA_RETRASO_FIN THEN
					CONTA_RETRASO <= 0;
					CONTA_UNIDADES <= CONTA_UNIDADES+1;
						IF CONTA_UNIDADES = 9 THEN--------------CUANDO EL CONTEO DE LAS UNIDADES LLEGE A 9
							CONTA_UNIDADES <= 0;-----------------REINICIA EL CONTADOR DE LAS UNIDADES
							CONTA_DECENAS <= CONTA_DECENAS +1;---E INCREMENTA EN 1 LAS DECENAS
								IF CONTA_DECENAS = 9 THEN
									CONTA_DECENAS <= 0;
								END IF;
						END IF;
					END IF;
			END IF;
		END IF;
	END IF;
END PROCESS;
--------------------------------------------------------------------


---MAQUINA DE ESTADOS QUE SE ENCARGA DEL SWITCHEO DE LOS TRANSISTORES-------
-------PARA ACTIVAR Y DESACTIVAR LOS DISPLAY CADA 10 MS---------------------
PROCESS(EDO_P)
BEGIN
	CASE EDO_P IS 
		WHEN UNIDADES =>
			EDO_F <= DECENAS;
			TRANSISTOR <= "01"; --ACTIVAMOS EL DISPLAY DE LAS UNIDADES
		WHEN DECENAS =>
			EDO_F <= UNIDADES;  --ACTIVAMOS EL DISPLAY DE LAS DECENAS
			TRANSISTOR <= "10";
		WHEN OTHERS => NULL;
	END CASE;
END PROCESS;


--CONTEO PARA UN RETRASO DE 10ms
PROCESS(CLK)
BEGIN
	IF RISING_EDGE(CLK) THEN
		CONTA_SWITCH <= CONTA_SWITCH +1;
		IF CONTA_SWITCH = CONTA_SW_FIN THEN
			CONTA_SWITCH <= 0;
			EDO_P <= EDO_F;
		END IF;
	END IF;
END PROCESS;


CONTADOR_PRINCIPAL <= CONTA_UNIDADES WHEN EDO_P = UNIDADES ELSE--MANDAMOS EL VALOR DE LAS UNIDADES CUANDO EDO_P ESTE EN UNIDADES
							 CONTA_DECENAS;									--MANDAMOS EL VALOR DE LAS DECENAS  CUANDO EDO_P ESTE EN DECENAS	
-----------------------------------------------------------------------------

----CODIFICADOR---

--VECTOR DE SALIDA CONECTADO A LOS DISPLAY DE CTODO COMN
--EN CASO DE USAR DISPLAYs DE NODO COMN SE TIENE QUE NEGAR EL VECTOR
--POR EJEMPLO: NOT "1111110"

          --ABCDEFG  
DISPLAY <= "1111110" WHEN CONTADOR_PRINCIPAL = 0 ELSE --0
			  "0110000" WHEN CONTADOR_PRINCIPAL = 1 ELSE --1
			  "1101101" WHEN CONTADOR_PRINCIPAL = 2 ELSE --2
			  "1111001" WHEN CONTADOR_PRINCIPAL = 3 ELSE --3
			  "0110011" WHEN CONTADOR_PRINCIPAL = 4 ELSE --4
			  "1011011" WHEN CONTADOR_PRINCIPAL = 5 ELSE --5
			  "1011111" WHEN CONTADOR_PRINCIPAL = 6 ELSE --6
			  "1110000" WHEN CONTADOR_PRINCIPAL = 7 ELSE --7
			  "1111111" WHEN CONTADOR_PRINCIPAL = 8 ELSE --8
			  "1111011";

end Behavioral;

